/*******************************************************************************

  Copyright(c) 2003-2004 Aurelien Reynaud.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The full GNU General Public License is included in this distribution in the
  file called COPYING.

*******************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "ggiterm.h"
#include "debug.h"


/*
 * Translates charcode to the corresponding graphical character when altcharset
 * mode is on.
 * This is the basic version, limited to ASCII characters.
 * Returns the translated character
 */
wchar_t ascii_get_altchar (wchar_t charcode)
{
	debug (DEBUG_FUNCTION, "called with args (charcode=%d)", (int)charcode);
	switch (charcode) {
		case '+': charcode = '>'; break; /* vt100 '+': arrow pointing right */
		case ',': charcode = '<'; break; /* vt100 ',': arrow pointing left */
		case '-': charcode = 0136; break; /* vt100 '-': arrow pointing up */
		case '.': charcode = 'v'; break; /* vt100 '.': arrow pointing down */
		case '0': charcode = '#'; break; /* vt100 '0': solid square block */
		case '`': charcode = '+'; break; /* vt100 '`': diamond */
		case 'a': charcode = ':'; break; /* vt100 'a': checker board (stipple) */
		case 'f': charcode = '\\'; break; /* vt100 'f': degree */
		case 'g': charcode = '#'; break; /* vt100 'g': plus/minus */
		case 'h': charcode = '#'; break; /* vt100 'h': board os squares */
		case 'j': charcode = '+'; break; /* vt100 'j': rightdown corner */
		case 'k': charcode = '+'; break; /* vt100 'k': upright corner */
		case 'l': charcode = '+'; break; /* vt100 'l': upleft corner */
		case 'm': charcode = '+'; break; /* vt100 'm': downleft corner */
		case 'n': charcode = '+'; break; /* vt100 'n': crossover */
		case 'o': charcode = '~'; break; /* vt100 'o': scan line 1 */
		case 'p': charcode = '-'; break; /* vt100 'p': scan line 3 */
		case 'q': charcode = '-'; break; /* vt100 'q': horizontal line */
		case 'r': charcode = '-'; break; /* vt100 'r': scan line 7 */
		case 's': charcode = '_'; break; /* vt100 's': scan line 9 */
		case 't': charcode = '+'; break; /* vt100 't': tee pointing right */
		case 'u': charcode = '+'; break; /* vt100 'u': tee pointing left */
		case 'v': charcode = '+'; break; /* vt100 'v': tee pointing up */
		case 'w': charcode = '+'; break; /* vt100 'w': tee pointing down */
		case 'x': charcode = '|'; break; /* vt100 'x': vertical line */
		case 'y': charcode = '<'; break; /* vt100 'y': less or equal */
		case 'z': charcode = '>'; break; /* vt100 'z': more or equal */
		case '{': charcode = '*'; break; /* vt100 '{': Pi */
		case '|': charcode = '!'; break; /* vt100 '|': not equal */
		case '}': charcode = 'f'; break; /* vt100 '}': UK pound */
		case '~': charcode = 'o'; break; /* vt100 '~': bullet */
	}
	debug (DEBUG_FUNCTION, "Leaving");
	return charcode;
}


#ifdef HAVE_FREETYPE

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_GLYPH_H

extern FT_Face face;

/*
 * Translates charcode to the corresponding graphical character when altcharset
 * mode is on.
 * This is the enhanced version, using all the possibilities of Unicode.
 * Fallbacks to the basic version if the best character does not exist in the
 * current font.
 * Returns the translated character
 */
wchar_t freetype_get_altchar (wchar_t charcode)
{
	wchar_t saved_charcode;
	
	debug (DEBUG_FUNCTION, "called with args (charcode=%d)", (int)charcode);
	
	saved_charcode = charcode;
	switch (charcode) {
		case '+': charcode = 0x2192; break; /* vt100 '+': arrow pointing right */
		case ',': charcode = 0x2190; break; /* vt100 ',': arrow pointing left */
		case '-': charcode = 0x2191; break; /* vt100 '-': arrow pointing up */
		case '.': charcode = 0x2193; break; /* vt100 '.': arrow pointing down */
		case '0': charcode = 0x2588; break; /* vt100 '0': solid square block */
		case '`':                              /* vt100 '`': diamond */
			charcode = 0x25C6;
			if (!FT_Get_Char_Index (face, charcode))
				charcode = 0x266;
			break;
		case 'a': charcode = 0x2592; break; /* vt100 'a': checker board (stipple) */
		case 'f': charcode = 0x00B0; break; /* vt100 'f': degree */
		case 'g':                            /* vt100 'g': plus/minus */
			charcode = 0x2213;
			if (!FT_Get_Char_Index (face, charcode))
				charcode = 0x00B1;
			break;
		case 'h': charcode = 0x25A6; break; /* vt100 'h': board of squares */
		case 'j': charcode = 0x2518; break; /* vt100 'j': rightdown corner */
		case 'k': charcode = 0x2510; break; /* vt100 'k': upright corner */
		case 'l': charcode = 0x250C; break; /* vt100 'l': upleft corner */
		case 'm': charcode = 0x2514; break; /* vt100 'm': downleft corner */
		case 'n': charcode = 0x253C; break; /* vt100 'n': crossover */
		case 'o': charcode = 0x23BA; break; /* vt100 'o': scan line 1 */
		case 'p': charcode = 0x23BB; break; /* vt100 'p': scan line 3 */
		case 'q': charcode = 0x2500; break; /* vt100 'q': horizontal line */
		case 'r': charcode = 0x23BC; break; /* vt100 'r': scan line 7 */
		case 's': charcode = 0x23BD; break; /* vt100 's': scan line 9 */
		case 't': charcode = 0x251C; break; /* vt100 't': tee pointing right */
		case 'u': charcode = 0x2524; break; /* vt100 'u': tee pointing left */
		case 'v': charcode = 0x2534; break; /* vt100 'v': tee pointing up */
		case 'w': charcode = 0x252C; break; /* vt100 'w': tee pointing down */
		case 'x': charcode = 0x2502; break; /* vt100 'x': vertical line */
		case 'y': charcode = 0x2264; break; /* vt100 'y': less or equal */
		case 'z': charcode = 0x2265; break; /* vt100 'z': more or equal */
		case '{': charcode = 0x03C0; break; /* vt100 '{': Pi */
		case '|': charcode = 0x2260; break; /* vt100 '|': not equal */
		case '}': charcode = 0x00A3; break; /* vt100 '}': UK pound */
		case '~': charcode = 0x2022; break; /* vt100 '~': bullet */
	}
	if (!FT_Get_Char_Index (face, charcode))
		charcode = ascii_get_altchar (saved_charcode);
	
	debug (DEBUG_FUNCTION, "Leaving");
	return charcode;
}

#endif /* HAVE_FREETYPE */
