/*******************************************************************************

  Copyright(c) 2003-2004 Aurelien Reynaud.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The full GNU General Public License is included in this distribution in the
  file called COPYING.

*******************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "ggiterm.h"
#include "debug.h"

#include <sys/ioctl.h>

/* GGI */
#include <ggi/ggi.h>
#ifdef HAVE_LIBGGIWMH
# include <ggi/wmh.h>
#endif

/* Global variables */
ggi_visual_t vis;
int visible_w, visible_h;
/*int virtual_w, virtual_h;*/
int cell_w, cell_h;
ggi_pixel color[16];
ggi_color palette[16] = {
	{ 0x0000, 0x0000, 0x0000, 0xFFFF },
	{ 0xB000, 0x0000, 0x0000, 0xFFFF },
	{ 0x0000, 0xB000, 0x0000, 0xFFFF },
	{ 0xB000, 0xB000, 0x0000, 0xFFFF },
	{ 0x0000, 0x0000, 0xB000, 0xFFFF },
	{ 0xB000, 0x0000, 0xB000, 0xFFFF },
	{ 0x0000, 0xB000, 0xB000, 0xFFFF },
	{ 0xB000, 0xB000, 0xB000, 0xFFFF },
	
	{ 0x1000, 0x1000, 0x1000, 0xFFFF },
	{ 0xFFFF, 0x0000, 0x0000, 0xFFFF },
	{ 0x0000, 0xFFFF, 0x0000, 0xFFFF },
	{ 0xFFFF, 0xFFFF, 0x0000, 0xFFFF },
	{ 0x0000, 0x0000, 0xFFFF, 0xFFFF },
	{ 0xFFFF, 0x0000, 0xFFFF, 0xFFFF },
	{ 0x0000, 0xFFFF, 0xFFFF, 0xFFFF },
	{ 0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF }
	};


void (*render_write_char) (int, int, cell_t, int);
void (*render_blink_char) (int, int, cell_t, int);
void (*render_delete_chars) (int, int, unsigned int);
void (*render_insert_chars) (int, int, unsigned int);
void (*render_erase_chars) (int, int, unsigned int);
void (*render_erase_line_from) (int, int);
void (*render_erase_line_to) (int, int);
void (*render_erase_line) (int);
void (*render_scroll_up) (int);
void (*render_scroll_up_region) (int, int, int);
void (*render_scroll_down_region) (int, int, int);
void (*render_erase_screen_from) (int, int);
void (*render_erase_screen_to) (int, int);
void (*render_erase_screen) ();
void (*render_flush) ();
void (*render_flush_area) (int, int, int, int);
void (*render_flash) ();

/* External functions */
int backend_init (char *, int, int*, int*);
void backend_exit ();
void render_load_copybox ();
void render_load_setorigin ();

#ifdef DEBUG
void show_grid (void)
{
	int x, y;
	
	debug (DEBUG_FUNCTION, "called with no args");
	
	ggiSetGCForeground (vis, color[2]);
	for (x = 0; x < visible_w; x += cell_w) {
		ggiDrawVLine (vis, x, 0, visible_h);
	}
	for (y = 0; y < visible_h; y += cell_h) {
		ggiDrawHLine (vis, 0, y, visible_w);
	}
	debug (DEBUG_FUNCTION, "Leaving");
}
#endif /* DEBUG */

void ggi_map_colors ()
{
	int i;
	
	debug (DEBUG_FUNCTION, "called with no args");
	for (i=0; i<16; i++) color[i] = ggiMapColor (vis, palette+i);
	debug (DEBUG_FUNCTION, "Leaving");
}


int ggi_init (void)
{
	int err, render_method = 1;
	ggi_mode sug_mode, final_mode;
	ggi_flags flags;
	/*const ggi_pixelformat *mypixelformat;*/
	char dummy[64];

	debug (DEBUG_FUNCTION, "called with no args");

	if (ggiInit () < 0) {
		error ("Cannot initalize LibGGI!");
		debug (DEBUG_FUNCTION, "Leaving");
		return 0;
	}
	debug (DEBUG_INIT, "LibGGI initialization successful");

	vis = ggiOpen (NULL);
	if (vis == NULL) {
		error ("Couldn't open default visual!");
		ggiExit ();
		debug (DEBUG_FUNCTION, "Leaving");
		return 0;
	}
	debug (DEBUG_INIT, "Default visual opened");
	
	ggiSetFlags (vis, GGIFLAG_ASYNC);
	flags = ggiGetFlags (vis);
	if (flags & GGIFLAG_ASYNC) {
		debug (DEBUG_INIT, "Asynchronous mode: ON");
	} else {
		debug (DEBUG_INIT, "Asynchronous mode: OFF");
	}
	
	/* Get the default mode */
	err = ggiCheckGraphMode (vis, GGI_AUTO, GGI_AUTO, GGI_AUTO, GGI_AUTO,
				 GT_AUTO, &sug_mode);
	if (err) {
		error ("No graphics mode available!");
		ggiExit ();
		debug (DEBUG_FUNCTION, "Leaving");
		return 0;
	}
	debug (DEBUG_INIT, "Automatic graphics mode check succeeded");
	ggiSPrintMode (dummy, &sug_mode);
	debug (DEBUG_INIT, "Suggested mode is: %s", dummy);
	debug (DEBUG_INIT,
	       "Physical resolution: %dx%d mm",
	       sug_mode.size.x, sug_mode.size.y);
	visible_w = sug_mode.visible.x;
	visible_h = sug_mode.visible.y;
	/*virtual_w = sug_mode.virt.x;
	virtual_h = sug_mode.virt.y;*/

	final_mode = sug_mode;
	
#ifdef FORCE_RENDER_COPYBOX
	debug (DEBUG_INIT, "CopyBox rendering forced at compile time");
#else
	/* Try virtual height = visible_height*2 to enable fast scrolling */
	debug (DEBUG_INIT, "Negociating mode suitable for SetOrigin rendering");
	err = ggiCheckGraphMode (vis, visible_w, visible_h, visible_w, visible_h*2,
				 GT_AUTO, &final_mode);
	if (!err) {
		debug (DEBUG_INIT, "Success: SetOrigin rendering possible");
		/*virtual_h = final_mode.virt.y;*/
		render_method = 2;
	} else {
		debug (DEBUG_INIT, "Failed: Back to CopyBox rendering mode");
		final_mode = sug_mode;
	}
	ggiSPrintMode (dummy, &final_mode);
	debug (DEBUG_INIT, "Negociated mode is: %s", dummy);
#endif

#ifdef HAVE_LIBGGIWMH
	err = ggiWmhInit ();
	if (!err) {
		debug (DEBUG_INIT, "Libggiwmh initialized");
		err = ggiWmhAttach (vis);
		if (!err) {
			debug (DEBUG_INIT, "Attached WMH extension");
		} else {
			debug (DEBUG_INIT, "Couldn't attach WMH extension");
		}
	} else {
		debug (DEBUG_INIT, "Couldn't initialize libggiwmh");
	}
#else
	debug (DEBUG_INIT, "Libggiwmh support not compiled in");
#endif
	
	debug (DEBUG_INIT, "Setting graphics mode");
	err = ggiSetMode (vis, &final_mode);
	if (err) {
		error ("Set mode failed although check was OK!");
		ggiExit ();
		debug (DEBUG_FUNCTION, "Leaving");
		return 0;
	}
	
	/*mypixelformat = ggiGetPixelFormat (vis);
	debug (DEBUG_INIT, "Pixel depth = %d",
	                   mypixelformat->depth);
	debug (DEBUG_INIT, "Pixel size = %d",
	                   mypixelformat->size);
	debug (DEBUG_INIT, "Pixel red shift = %d",
	                   mypixelformat->red_shift);
	debug (DEBUG_INIT, "Pixel red mask = 0x%08lX",
	                   (unsigned long)mypixelformat->red_mask);
	debug (DEBUG_INIT, "Pixel green shift = %d",
	                   mypixelformat->green_shift);
	debug (DEBUG_INIT, "Pixel green mask = 0x%08lX",
	                   (unsigned long)mypixelformat->green_mask);
	debug (DEBUG_INIT, "Pixel blue shift = %d",
	                   mypixelformat->blue_shift);
	debug (DEBUG_INIT, "Pixel blue mask = 0x%08lX",
	                   (unsigned long)mypixelformat->blue_mask);
	debug (DEBUG_INIT, "Pixel alpha shift = %d",
	                   mypixelformat->alpha_shift);
	debug (DEBUG_INIT, "Pixel alpha mask = 0x%08lX",
	                   (unsigned long)mypixelformat->alpha_mask);
	debug (DEBUG_INIT, "Pixel clut shift = %d",
	                   mypixelformat->clut_shift);
	debug (DEBUG_INIT, "Pixel clut mask = 0x%08lX",
	                   (unsigned long)mypixelformat->clut_mask);
	debug (DEBUG_INIT, "Pixel fg shift = %d",
	                   mypixelformat->fg_shift);
	debug (DEBUG_INIT, "Pixel fg mask = 0x%08lX",
	                   (unsigned long)mypixelformat->fg_mask);
	debug (DEBUG_INIT, "Pixel bg shift = %d",
	                   mypixelformat->bg_shift);
	debug (DEBUG_INIT, "Pixel bg mask = 0x%08lX",
	                   (unsigned long)mypixelformat->bg_mask);
	debug (DEBUG_INIT, "Pixel texture shift = %d",
	                   mypixelformat->texture_shift);
	debug (DEBUG_INIT, "Pixel texture mask = 0x%08lX",
	                   (unsigned long)mypixelformat->texture_mask);*/

	debug (DEBUG_INIT, "Setting color palette");
	ggi_map_colors ();
	
	ggiSetGCForeground (vis, color[0]);
	ggiFillscreen (vis);

	debug (DEBUG_FUNCTION, "Leaving");
	return render_method;
}

void ggi_exit (void)
{
	debug (DEBUG_FUNCTION, "called with no args");
#ifdef HAVE_LIBGGIWMH
	ggiWmhDetach (vis);
	ggiWmhExit ();
#endif
	ggiClose (vis);
	ggiExit ();
	debug (DEBUG_FUNCTION, "Leaving");
}

int lowlevel_init (char *font_file, int fontsize, char *title, int *terminal_w, int *terminal_h)
{
	int err, render_mode;
	
	debug (DEBUG_FUNCTION,
	       "called with args (font_file=%s, fontsize=%d, title=%s, *terminal_w, *terminal_h",
	       font_file, fontsize, title);

	debug (DEBUG_INIT, "Initializing GGI");
	render_mode = ggi_init ();
	if (!render_mode) {
			error ("Cannot complete GGI setup. Aborting.");
			debug (DEBUG_FUNCTION, "Leaving");
			return 1;
	}
	debug (DEBUG_INIT, "GGI setup complete");

	debug (DEBUG_INIT, "Initializing backend");
	err = backend_init (font_file, fontsize, &cell_w, &cell_h);
	if (err) {
		error ("Cannot initialize backend. Aborting.");
		ggi_exit ();
		debug (DEBUG_FUNCTION, "Leaving");
		return 1;
	}
	debug (DEBUG_INIT, "Backend initialization complete");
	debug (DEBUG_INIT, "Characters are %d x %d pixels", cell_w, cell_h);
	
	switch (render_mode) {
		case 1:
			debug (DEBUG_INIT, "Loading CopyBox renderers");
			render_load_copybox ();
			break;
		case 2:
			debug (DEBUG_INIT, "Loading SetOrigin renderers");
			render_load_setorigin ();
			break;
	}
	
	*terminal_w = visible_w / cell_w;
	*terminal_h = visible_h / cell_h;
#ifdef DEBUG
	if (debuglevel) {
		show_grid ();
	}
#endif /* DEBUG */
	
#ifdef HAVE_LIBGGIWMH
	ggiWmhSetTitle (vis, title);
	ggiWmhSetIconTitle (vis, title);
	/*err = ggiWmhGetRootSize (vis, &rootw, &rooth, GGIWMH_ROOTWIN_DEFAULT);
	debug (DEBUG_INIT, "Root window size is %dx%d", rootw, rooth);
	err = ggiWmhAllowResize (vis, cell_w, cell_h, rootw, rooth, cell_w, cell_h);
	if (!err) {
		debug (DEBUG_INIT, "Resize feature: AVAILABLE");
	} else {
		debug (DEBUG_INIT, "Resize feature: NOT AVAILABLE");
	}*/
#endif

	debug (DEBUG_FUNCTION, "Leaving");
	return 0;
}

void lowlevel_exit ()
{
	debug (DEBUG_FUNCTION, "called with no args");
	backend_exit ();
	ggi_exit ();
	debug (DEBUG_FUNCTION, "Leaving");
}

#ifndef DISABLE_RESIZE
int lowlevel_resize (int new_w, int new_h)
{
	int old_visible_w, old_visible_h;
	int err;
	int w, h;
	struct winsize ws;
	ggi_mode mode;
	
	debug (DEBUG_FUNCTION, "Entering");
	
	set_cursor_status (CURSOR_HIDE);
	
	old_visible_w = visible_w;
	old_visible_h = visible_h;
	visible_w = new_w;
	visible_h = new_h;
	debug (DEBUG_INIT, "Set visible area to %dx%d", visible_w, visible_h);
	
	if (fast_scroll_enabled) {
		virtual_w = new_w;
		virtual_h = new_h*2;
	} else {
		virtual_w = new_w;
		virtual_h = new_h;
	}
	debug (DEBUG_INIT, "Set virtual area to %dx%d", virtual_w, virtual_h);
	debug (DEBUG_INIT, "Set current_origin to %d", current_origin);
	
	ggiGetMode (vis, &mode);
	mode.visible.x = visible_w;
	mode.visible.y = visible_h;
	mode.virt.x = virtual_w;
	mode.virt.y = virtual_h;
	ggiSetMode (vis, &mode);
	
	err = terminal_resize (visible_w/cell_w, visible_h/cell_h);
	
	ggiFlush (vis);
	
	terminal_get_size (&w, &h);
	ws.ws_col = w;
	ws.ws_row = h;
	ws.ws_xpixel = visible_w;
	ws.ws_ypixel = visible_h;
	if (ioctl (terminal_fd, TIOCSWINSZ, &ws) < 0) {
		error ("Error setting window size\n");
	}
	
	debug (DEBUG_FUNCTION, "Leaving");
	return 1;
}
#endif /* DISABLE_RESIZE */
