/*******************************************************************************

  Copyright(c) 2003-2004 Aurelien Reynaud.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The full GNU General Public License is included in this distribution in the
  file called COPYING.

*******************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "ggiterm.h"
#include "debug.h"
#include "terminal.h"
#include <ggi/ggi.h>


/* External variables */
extern ggi_visual_t vis;
extern ggi_pixel color[];

/* External functions */
wchar_t ascii_get_altchar (wchar_t);

int native_height, native_width;

int native_write_char (int x, int y, cell_t charcode, int blinking)
{
	int cur_bgcolor, cur_fgcolor, dummy_color;
	
	debug (DEBUG_FUNCTION,
	       "called with args (x=%d, y=%d, charcode=[cell_t], blinking=%d)",
	       x, y, blinking);
	
	cur_bgcolor = charcode.bgcolor;
	cur_fgcolor = charcode.fgcolor;
	if (charcode.mode & MODE_ALTCHARSET) {
		charcode.charcode = ascii_get_altchar (charcode.charcode);
	}
	if (charcode.mode & MODE_REVERSE) {
		dummy_color = cur_bgcolor;
		cur_bgcolor = cur_fgcolor;
		cur_fgcolor = dummy_color;
	}
	if (charcode.mode & MODE_BOLD) {
		cur_fgcolor += 8;
	}
	if (charcode.mode & MODE_INVISIBLE) {
		cur_fgcolor = cur_bgcolor;
	}
	if ((charcode.mode & MODE_BLINK) && blinking) {
		dummy_color = cur_bgcolor;
		cur_bgcolor = cur_fgcolor;
		cur_fgcolor = dummy_color;
	}
	
	ggiSetGCBackground (vis, color[cur_bgcolor]);
	ggiSetGCForeground (vis, color[cur_fgcolor]);
	ggiPutc (vis, x, y, (char) charcode.charcode);
	
	ggiSetGCForeground (vis, color[cur_bgcolor]);
	ggiDrawHLine (vis, x, y+native_height-1, native_width);
	if (charcode.mode & MODE_UNDERLINE) {
		ggiSetGCForeground (vis, color[cur_fgcolor]);
	}
	ggiDrawHLine (vis, x, y+native_height-2, native_width);
	debug (DEBUG_FUNCTION, "Leaving");
	return 0;
}


int native_init (int *cell_w, int *cell_h)
{
	debug (DEBUG_FUNCTION, "called with args (*cell_w, *cell_h)");
	
	ggiGetCharSize (vis, &native_width, &native_height);
	debug (DEBUG_INIT, "Native character size %d x %d",
	       native_width, native_height);
	/* Add some space below for underlining */
	native_height += 2;
	debug (DEBUG_INIT, "Using character size  %d x %d",
	       native_width, native_height);
	*cell_w = native_width;
	*cell_h = native_height;

	debug (DEBUG_FUNCTION, "Leaving");
	return 0;
}

wchar_t native_get_charmap ()
{
	static wchar_t last = 0;
	
	debug (DEBUG_FUNCTION, "called with no args");
	if (last == 0) {
		last = 32;
	} else {
		last++;
		if (last == 127) {
			last = 0;
		}
	}
	debug (DEBUG_FUNCTION, "Leaving");
	return last;
}

void native_exit ()
{
	debug (DEBUG_FUNCTION, "called with no args");
	debug (DEBUG_FUNCTION, "Leaving");
}
