/*******************************************************************************

  Copyright(c) 2003-2004 Aurelien Reynaud.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The full GNU General Public License is included in this distribution in the
  file called COPYING.

*******************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "ggiterm.h"
#include "debug.h"
#include "lowlevel/lowlevel.h"
#include "history.h"


extern int terminal_height;
extern int blinking_state;

extern int linewrap_pending;
extern int cursor_x, cursor_y;
void cursor_advance ();
void cursor_move (int, int);


wchar_t last_char;
int mode = 0;
int fgcolor, bgcolor;


void text_set_mode (int mask)
{
	debug (DEBUG_FUNCTION, "called with no args");
	mode |= mask;
	debug (DEBUG_FUNCTION, "Leaving");
}

void text_unset_mode (int mask)
{
	debug (DEBUG_FUNCTION, "called with no args");
	mode &= ~mask;
	debug (DEBUG_FUNCTION, "Leaving");
}

void text_set_bgcolor (unsigned int color)
{
	debug (DEBUG_FUNCTION, "called with no args");
	bgcolor = color == 9 ? default_bgcolor : color;
	debug (DEBUG_FUNCTION, "Leaving");
}

void text_set_fgcolor (unsigned int color)
{
	debug (DEBUG_FUNCTION, "called with no args");
	fgcolor = color == 9 ? default_fgcolor : color;
	debug (DEBUG_FUNCTION, "Leaving");
}

void text_write_char (wchar_t charcode)
{
	cell_t current_char;
	
	debug (DEBUG_FUNCTION, "called with args (charcode=0x%X)", (int)charcode);
	current_char.charcode = charcode;
	current_char.bgcolor = bgcolor;
	current_char.fgcolor = fgcolor;
	current_char.mode = mode;
	if (linewrap_pending) {
		cursor_move (0, cursor_y + 1);
	}
	history_write_char (cursor_x, cursor_y, current_char);
	render_write_char (cursor_x, cursor_y, current_char, blinking_state);
	last_char = charcode;
	cursor_advance ();
	debug (DEBUG_FUNCTION, "Leaving");
}

/* Repeat last character count times */
void text_repeat_char (unsigned int count)
{
	debug (DEBUG_FUNCTION, "called with args (count=%d)", count);
	while (count) {
		text_write_char (last_char);
		count--;
	}
	debug (DEBUG_FUNCTION, "Leaving");
}

void text_write_area (int x, int y, int w, int h)
{
	int i, j, length;
	cell_t dummy;
	
	debug (DEBUG_FUNCTION, "called with args (x=%d, y=%d, w=%d, h=%d)", x, y, w, h);
	for (j=y; j<(y+h<terminal_height?y+h:terminal_height); j++) {
		length = history_get_line_length (j);
		for (i=x; i<(length<x+w?length:x+w); i++) {
			dummy = history_get_char(i, j);
			render_write_char (i, j, dummy, blinking_state);
		}
		render_erase_chars (i, j, x + w - i);
	}
	debug (DEBUG_FUNCTION, "Leaving");
}
